package jugs.javafx.b_properties;

import javafx.application.Application;
import javafx.collections.FXCollections;
import javafx.concurrent.Task;
import javafx.geometry.HPos;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.control.ProgressBar;
import javafx.scene.control.TextField;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.GridPane;
import javafx.scene.paint.Color;
import javafx.stage.Stage;

/**
 * Beispielprogramm für JavaFX-Schulung
 * 
 * @author Michael Inden
 * 
 * Copyright 2015 by Michael Inden 
 */
public class Exercise2_ProgressBarAndTaskExample extends Application
{
    private Task<?> copyWorker;

    // Common shared Controls
    private final ProgressBar progressBar = new ProgressBar(0);
    private final TextField message = new TextField();
    
    private final Button startButton = new Button("Start");
    private final Button cancelButton = new Button("Cancel");
    
    
    @Override
    public void start(final Stage primaryStage)
    {
        final Label label1 = new Label("Duration:");
        final ComboBox<Integer> durationComboBox = new ComboBox<>(FXCollections.observableArrayList(100, 200, 500,
                                                                                                    1000, 2000, 5000));
//        durationComboBox.getSelectionModel().select(500); // => int => index 500
//        durationComboBox.getSelectionModel().select(Integer.valueOf(500));

        final Label label1b = new Label("Steps:");
        final ChoiceBox<Integer> cb = new ChoiceBox<>();
        cb.getItems().addAll(10, 20, 30, 50, 100);
        cb.getSelectionModel().select(Integer.valueOf(30));
        
        final GridPane gridPane = new GridPane();
        gridPane.setPadding(new Insets(5));
        gridPane.setVgap(5);
        gridPane.setHgap(5);
        gridPane.add(label1, 0, 0);
        gridPane.add(durationComboBox, 1, 0);
        gridPane.add(label1b, 2, 0);
        gridPane.add(cb, 3, 0);
        gridPane.add(new Label("Progress:"), 0, 1);
        gridPane.add(progressBar, 1, 1);
        gridPane.add(new Label("Message:"), 0, 2);
        gridPane.add(message, 1, 2);
        GridPane.setHalignment(progressBar, HPos.CENTER);
        progressBar.setMaxWidth(Double.MAX_VALUE);
        GridPane.setColumnSpan(progressBar, 3);
        GridPane.setColumnSpan(message, 3);
        
        final FlowPane flowPane = new FlowPane();
        flowPane.setPadding(new Insets(7));
        flowPane.setHgap(7);
        flowPane.setAlignment(Pos.BOTTOM_RIGHT);
        flowPane.getChildren().addAll(startButton, cancelButton);

        startButton.setOnAction(event -> {
            final int durationsInMs = durationComboBox.getValue();
            final int steps = cb.getValue();
            onStart(durationsInMs, steps);
        });
        cancelButton.setOnAction(event -> {
            onCancel();
        });
        
        
        final BorderPane mainPane = new BorderPane();
        mainPane.setTop(gridPane);
        mainPane.setBottom(flowPane);

        final Scene scene = new Scene(mainPane, 400, 150, Color.WHITE);
        primaryStage.setTitle("ProgressBarAndTaskExample");
        primaryStage.setScene(scene);
        primaryStage.show();
    }

    protected void onStart(final int durationsInMs, final int steps)
    {
        startButton.setDisable(true);
        cancelButton.setDisable(false);

        copyWorker = createWorker(durationsInMs, steps);

        progressBar.progressProperty().unbind();
        progressBar.setProgress(0);
        
        progressBar.progressProperty().bind(copyWorker.progressProperty());
        message.textProperty().bind(copyWorker.messageProperty());

        new Thread(copyWorker).start();
    }

    protected void onCancel()
    {
        startButton.setDisable(false);
        cancelButton.setDisable(true);
        
        copyWorker.cancel(true);
        
        progressBar.progressProperty().unbind();
        message.textProperty().unbind();        
    }

    private Task<Integer> createWorker(int durationsInMs, int steps)
    {
        // TODO
        throw new UnsupportedOperationException("Not implemented yet!");
    }
    
    public static void main(String[] args)
    {
        Application.launch(args);
    }
}
