package jugs.javafx.c_tables;

import java.util.Arrays;
import java.util.List;

import javafx.application.Application;
import javafx.scene.Scene;
import javafx.scene.control.TreeItem;
import javafx.scene.control.TreeTableCell;
import javafx.scene.control.TreeTableColumn;
import javafx.scene.control.TreeTableView;
import javafx.scene.control.cell.TreeItemPropertyValueFactory;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;
import javafx.util.Callback;

/**
 * Beispielprogramm zeigt die mit JDK 8 eingeführte DatePicker-Komponente mit eigenem Renderer.
 * 
 * @author Michael Inden
 * 
 * Copyright 2014 by Michael Inden 
 */
public class Exercise4_TreeTableViewExample extends Application
{
    public void start(final Stage stage)
    {
        final TreeItem<SimplePerson> root = createTreeData();

        final List<TreeTableColumn<SimplePerson, ?>> columns = createColumns();

        final TreeTableView<SimplePerson> treeTableView = new TreeTableView<SimplePerson>(root);
        treeTableView.getColumns().addAll(columns);

        final VBox vbox = new VBox();
        vbox.getChildren().addAll(treeTableView);

        stage.setScene(new Scene(vbox, 300, 250));
        stage.setTitle("TreeTableViewExample");
        stage.show();
    }

    private TreeItem<SimplePerson> createTreeData()
    {
        final TreeItem<SimplePerson> root = createGroupTreeItem("All Persons");

        final TreeItem<SimplePerson> group1 = createGroupTreeItem("Group 1");
        final TreeItem<SimplePerson> childNode1_1 = createPersonTrreItem("Micha", 43, 184);
        final TreeItem<SimplePerson> childNode1_2 = createPersonTrreItem("Tom", 22, 177);
        final TreeItem<SimplePerson> childNode1_3 = createPersonTrreItem("Lili", 34, 170);

        final TreeItem<SimplePerson> group2 = createGroupTreeItem("Group 2");
        final TreeItem<SimplePerson> childNode2_1 = createPersonTrreItem("Tim", 43, 181);
        final TreeItem<SimplePerson> childNode2_2 = createPersonTrreItem("Jens", 47, 175);
        final TreeItem<SimplePerson> childNode2_3 = createPersonTrreItem("Andy", 31, 178);

        group1.getChildren().setAll(childNode1_1, childNode1_2, childNode1_3);
        group2.getChildren().setAll(childNode2_1, childNode2_2, childNode2_3);
        root.getChildren().setAll(group1, group2);

        root.setExpanded(true);
        group1.setExpanded(true);

        return root;
    }

    private TreeItem<SimplePerson> createGroupTreeItem(final String groupName)
    {
        return new TreeItem<>(new SimplePerson(groupName, null, -1));
    }

    private TreeItem<SimplePerson> createPersonTrreItem(final String name, final int age, final int size)
    {
        return new TreeItem<>(new SimplePerson(name, age, size));
    }

    private <V> TreeTableColumn<SimplePerson, V> createColumn(final String columnTitle, final String attributeName,
                                                        final int prefWidth)
    {
        final TreeTableColumn<SimplePerson, V> column = new TreeTableColumn<>(columnTitle);
        column.setPrefWidth(prefWidth);

        column.setCellValueFactory(new TreeItemPropertyValueFactory<SimplePerson, V>(attributeName));

        return column;
    }

    private List<TreeTableColumn<SimplePerson, ?>> createColumns()
    {
        final TreeTableColumn<SimplePerson, Integer> sizeColumn = createColumn("Size", "size", 70);
        registerRenderer(sizeColumn, " cm");

        return Arrays.asList(createColumn("Name", "name", 125), createColumn("Age", "age", 50), sizeColumn);
    }

    private void registerRenderer(final TreeTableColumn<SimplePerson, Integer> sizeColumn, final String unitPostfix)
    {
        sizeColumn.setCellFactory(new Callback<TreeTableColumn<SimplePerson, Integer>, TreeTableCell<SimplePerson, Integer>>()

        {
            @Override
            public TreeTableCell<SimplePerson, Integer> call(final TreeTableColumn<SimplePerson, Integer> p)
            {
                return new TreeTableCell<SimplePerson, Integer>()
                {
                    @Override
                    protected void updateItem(final Integer item, final boolean empty)
                    {
                        super.updateItem(item, empty);

                        if (!empty)
                        {
                            if (item.intValue() == -1)
                            {
                                setText(null);
                            }
                            else
                            {
                                setText(item + unitPostfix);
                            }
                        }
                        else
                        {
                            // Wichtig, sonst werden Texte dargestellt, obwohl
                            // Parent zugeklappt ist!!!
                            setText(null);
                        }
                    }
                };
            }
        });
    }

    public static void main(final String[] args)
    {
        launch(args);
    }
}