package jugs.javafx.c_tables;

import java.time.LocalDate;

import javafx.beans.binding.Bindings;
import javafx.scene.control.ContentDisplay;
import javafx.scene.control.DatePicker;
import javafx.scene.control.TableCell;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;

/**
 * Beispielprogramm für JavaFX-Schulung 
 * 
 * @author Michael Inden
 * 
 * Copyright 2015 by Michael Inden 
 */
public class LocalDateCell extends TableCell<PersonEx, LocalDate>
{
    private final DatePicker        datePicker;

    public LocalDateCell()
    {
        datePicker = new DatePicker();

        datePicker.setOnAction( evt -> commitEdit(LocalDate.from(datePicker.getValue())) );
        datePicker.addEventFilter(KeyEvent.KEY_PRESSED, (KeyEvent event) -> {
            if (event.getCode() == KeyCode.ENTER || event.getCode() == KeyCode.TAB)
            {
                datePicker.setValue(datePicker.getConverter().fromString(datePicker.getEditor().getText()));
                commitEdit(LocalDate.from(datePicker.getValue()));
            }
            if (event.getCode() == KeyCode.ESCAPE)
            {
                cancelEdit();
            }
        });

        contentDisplayProperty().bind(Bindings.when(editingProperty()).then(ContentDisplay.GRAPHIC_ONLY)
                                                      .otherwise(ContentDisplay.TEXT_ONLY));
    }

    @Override
    public void updateItem(LocalDate birthday, boolean empty)
    {
        super.updateItem(birthday, empty);
        if (empty)
        {
            setText(null);
            setGraphic(null);
        }
        else
        {
            setText(birthday.toString());
            setGraphic(datePicker);
        }
    }

    @Override
    public void startEdit()
    {
        super.startEdit();
        if (!isEmpty())
        {
            datePicker.setValue(getLocalDate());
        }
    }

    private LocalDate getLocalDate() {
        return getItem() == null ? LocalDate.now() : getItem();
    }
}